PROGRAM dodd;
(* Duke of Dragonfear: Deathland.
 *	An action game for the 1st Pascal Game Development Mini Contest.
 *
 * Main program file. *)
(*
  Copyright (c) 2011 Guillermo Martínez J.

  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
    distribution.
*)

USES
  config, error, framectr, game, gamedata, graphsys, Lighting, title, usrInput,
  allegro, sysutils;



(* Initializes the program. *)
  FUNCTION InitProgram: BOOLEAN;
  BEGIN
    InitProgram := FALSE;
    TRY
    { Installs Allegro. }
      al_set_uformat (AL_U_UTF8);
      IF NOT al_init THEN
	RAISE Exception.Create ('Can''t initialize Allegro!');
    { Gets the configuration. }
      GetConfiguration;
    { Installs sound.  It's done before to set the graphic mode because on some
      systems (i.e. Windows) it swaps with console/desktop while detecting the
      sound system. }
	IF NOT al_install_sound (AL_DIGI_AUTODETECT, AL_MIDI_AUTODETECT) THEN
	  al_install_sound (AL_DIGI_AUTODETECT, AL_MIDI_NONE);
    { Sets the graphic mode. }
      InitGraphicMode;
      TRY
	al_set_window_title ('Duke of Dragonfear - Deathland');
	al_textout_centre_ex (ActivePage, al_font, 'Downgrading your system, please wait...',
	  ActivePage^.w DIV 2, ActivePage^.h DIV 2, al_makecol (255, 128, 128), -1);
	SwapVideoPages;
	framectr.Install; { It initialises Allegro's timer. }
      { Installs the other Allegro modules.  Does it after set the
	configuracion file because Allegro will use it too. }
	al_install_keyboard;
	al_install_joystick (AL_JOY_TYPE_AUTODETECT);
      { Inits user input. }
	UserInput[1] := TKeyboardInput.Create (1);
      { Loads the game data. }
	gamedata.Load;
      { Init lighting system. }
	Lighting.Init;
      { Initializes the random number generator. }
	RANDOMIZE;
      { Waits few seconds.  This is because some modern screens needs time to get
        synchronised. }
{ DISABLED FOR DEBUGGING.
	al_rest (2000);
}
      { Everything is Ok. }
	InitProgram := TRUE;
      EXCEPT
	ON E: Exception DO Error.Show (E.message);
      END;
    EXCEPT
      ON E: Exception DO WriteLn (E.message);
    END;
  END;



(* Releases all resources and closes down the program. *)
  PROCEDURE EndProgram;
  BEGIN
    Lighting.Shutdown;
    gamedata.Destroy;
    CloseGraphicMode;
  END;



(* The program starts here. *)
BEGIN
  IF NOT InitProgram THEN
    EXIT;
  TRY
{ The main loop. }
    WHILE Title.Run DO
      Game.Play;
  EXCEPT
    ON E: Exception DO Error.Show (E.message);
  END;
  EndProgram;
END.
