UNIT game;
(* Duke of Dragonfear: Deathland.
 *	An action game for the 1st Pascal Game Development Mini Contest.
 *
 * Main loop game.
 *)
(*
  Copyright (c) 2011 Guillermo Martínez J.

  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
    distribution.
*)

INTERFACE

(* Runs it. *)
  PROCEDURE Play;

IMPLEMENTATION

USES
  Actor, Enemies, error, framectr, GameData, graphsys, Lighting, mapmaker,
  Player, Sprites, Tilemap,
  allegro, sysutils;



(* Runs it. *)
  PROCEDURE Play;
  VAR
    EndLoop, EndPlayLoop: BOOLEAN;
    ScreenShootCnt: INTEGER;
    ScrollX, ScrollY: INTEGER;

  (* Helper procedure to create a new stage. *)
    PROCEDURE NewStage;
    BEGIN
    { Creates the level. }
      GetTilesetFromBitmap (GameData.DungeonTileset);
      mapmaker.CreateLevel;
    END;

  (* End secuence for the stage. *)
    PROCEDURE EndStage (PlayerWin: BOOLEAN);
    BEGIN
      //Error.Show ('End of stage');
    END;

  (* Helper procedure to take screen-shots. *)
    PROCEDURE ScreenShoot;
    BEGIN
      al_save_bitmap ('shoot'+IntToStr (ScreenShootCnt)+'.pcx', VisualPage, NIL);
      INC (ScreenShootCnt);
      al_key[AL_KEY_F12] := 0; { To avoid repetition. }
    { Save a bitmap is a long task, so restart the Tick variable. }
      Tick := 1;
    END;

  VAR
    Ndx: INTEGER;
  BEGIN
    TRY
      FadeOut (8);
    { Set up the play. }
      PlayerList := TActorList.Create;
      PlayerList.Add (TKnight.Create);
      PlayerList.Add (TTorchMan.Create);
      (PlayerList.Actor[1] AS TTorchMan).MainCharacter := PlayerList.Actor[0];
      EnemyList := TActorList.Create;
      EndLoop := FALSE;
    { Set up the screen shoot system. }
      ScreenShootCnt := 0;
    { The game loop. }
      REPEAT
      { Create the stage. }
	NewStage;
      { Play the game. }
	EndPlayLoop := FALSE;
	Tick := 1; { Must update the first frame. }
	REPEAT
	{ Logical loop. }
	  REPEAT
	    IF al_key[AL_KEY_ESC] <> 0 THEN
	    BEGIN
	      EndPlayLoop := TRUE;
	      EndLoop := TRUE;
	    END;
	    IF al_key[AL_KEY_F12] <> 0 THEN
	      ScreenShoot;
	  { Update game objects. }
	    PlayerList.Update;
	    EnemyList.Update;
	    DEC (Tick); { Frame updated. }
	  { If there are too much frames to update then it is too slow. }
{ DISABLED FOR DEBUG
	    IF Tick > 30 THEN
	    BEGIN
	      Error.Show ('Sorry but your computer is too slow.');
	      EndPlayLoop := TRUE;
	      EndLoop := TRUE;
	      BREAK;
	    END;
}
	  UNTIL Tick < 1; { While there are frames to uptade. }
	{ Graphics. }
	  (PlayerList.Actor[1] AS TPlayerActor).Scroll (ScrollX, ScrollY);
	  TileMap.Draw (ActivePage, ScrollX, ScrollY);
	  Lighting.Apply (ActivePage, ScrollX, ScrollY);
	  Sprites.DrawLighted (ActivePage, ScrollX, ScrollY);
	  FOR Ndx := 0 TO PlayerList.Count - 1 DO
	    (PlayerList.Actor[Ndx] AS TPlayerActor).DrawHealth (ActivePage, 10, 420 + (Ndx * 30));
	  SwapVideoPages;
	{ Next loop whaits until there are frames to update. }
	  WHILE Tick < 1 DO
	  { By calling "al_rest" we say to the OS that we don't need the CPU
	    for a while, so it can reduce CPU usage, saves battery power,
	    reduces fan noise, etcetera. }
	    al_rest (1);
	UNTIL EndPlayLoop;
      UNTIL EndLoop;
    FINALLY
      FreeAndNil (PlayerList);
      FreeAndNil (EnemyList);
      EndStage (TRUE);
    END;
  END;

END.

