UNIT Geometry;
(* Duke of Dragonfear: Deathland.
 *	An action game for the 1st Pascal Game Development Mini Contest.
 *
 * Some geometry.
 *)
(*
  Copyright (c) 2011 Guillermo Martínez J.

  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
    distribution.
*)

INTERFACE

(* Returns the square of the distance between two points.  You know:
   (h * h) = (c * c) + (C * C)

   It returns (h * h). *)
  FUNCTION Distance2 (X1, Y1, X2, Y2: INTEGER): INTEGER;

(* Returns the "angle" from <X1, Y1> to <X2, Y2>. *)
  FUNCTION LookAt (X1, Y1, X2, Y2: INTEGER): INTEGER;

IMPLEMENTATION

USES
  Actor, TileMap;



(* Returns the square of the distance between two points.  You know:
   (h * h) = (c * c) + (C * C)

   It returns (h * h). *)
  FUNCTION Distance2 (X1, Y1, X2, Y2: INTEGER): INTEGER;
  VAR
    X, Y: INTEGER;
  BEGIN
    X := ABS (X1 - X2);
    Y := ABS (Y1 - Y2);
    Distance2 := (X * X) + (Y * Y);
  END;



(* Returns the "angle" from <X1, Y1> to <X2, Y2>. *)
  FUNCTION LookAt (X1, Y1, X2, Y2: INTEGER): INTEGER;
  VAR
    X, Y, Tmp: INTEGER;
  BEGIN
    X := X2 - X1;
    Y := Y2 - Y1;
  { How much far the points are. }
    IF (ABS (Y) > TSIZE) OR (ABS (Y) > TSIZE) THEN
    BEGIN
    { It's not perfect. }
      IF ABS (X) < TSIZE DIV 3 THEN
	X := 0;
      IF ABS (Y) < TSIZE DIV 3 THEN
	Y := 0;
      IF Y < 0 THEN
	Tmp := RS_N
      ELSE IF Y > 0 THEN
	Tmp := RS_S
      ELSE
	Tmp := 0;
      IF X < 0 THEN
      BEGIN
	IF Tmp = RS_N THEN
	  LookAt := RS_NW
	ELSE IF Tmp = RS_S THEN
	  LookAt := RS_SW
	ELSE
	  LookAt := RS_W
      END
      ELSE IF X > 0 THEN
      BEGIN
	IF Tmp = RS_N THEN
	  LookAt := RS_NE
	ELSE IF Tmp = RS_S THEN
	  LookAt := RS_SE
	ELSE
	  LookAt := RS_E
      END
      ELSE
	LookAt := Tmp;
    END
    ELSE BEGIN
      IF ABS (Y) > ABS (X) THEN
      BEGIN
	IF Y < 0 THEN
	  LookAt := RS_N
	ELSE
	  LookAt := RS_S;
      END
      ELSE BEGIN
	IF X < 0 THEN
	  LookAt := RS_W
	ELSE
	  LookAt := RS_E;
      END;
    END;
  END;

END.
