UNIT Zombie;
(* Duke of Dragonfear: Deathland.
 *	An action game for the 1st Pascal Game Development Mini Contest.
 *
 * The Zombie.
 *)
(*
  Copyright (c) 2011 Guillermo Martínez J.

  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
    distribution.
*)

INTERFACE

USES
  actor;

TYPE
(* The Zombie actor. *)
  TZombie = CLASS (TActor)
  PRIVATE
  (* A counter with different uses depending on the player status. *)
    Counter: INTEGER;
  PUBLIC
  (* Creates the zombie. *)
    CONSTRUCTOR Create;
  (* The zombie inteligence... or someting. *)
    PROCEDURE Update; OVERRIDE;
  END;



IMPLEMENTATION

USES
  gamedata, Geometry, graphsys, Player, Sprites, TileMap;



CONST
  MAX_CAN_SEE_DISTANCE = 200;
  MAX_CAN_SEE_DISTANCE2 = MAX_CAN_SEE_DISTANCE * MAX_CAN_SEE_DISTANCE;


(***********
 * TZombie *
 ***********)

(* Creates the zombie. *)
  CONSTRUCTOR TZombie.Create;
  VAR
    Px, Py: INTEGER;
  BEGIN
    INHERITED Create (gamedata.Zombie, Sprites.NextFreeSprite);
    fTypeId := atEnemy;
    fStatus := aaStop;
    fDirection := Random (RS_SW) + 1;
    fFrame := RS_STAND;
  { At the moment puts the sprite randomly. }
    REPEAT
      Px := Random (TileMap.Width) + 1;
      PY := Random (TileMap.Height) + 1;
    UNTIL IsWalkable (Px, Py);
    WITH SpriteList[fSprIndex] DO
    BEGIN
      X := Px * TSIZE;
      Y := Py * TSIZE;
      OffX :=  -95;
      OffY := -127;
      Picture := fPictures.Picture[fDirection, fFrame];
    END;
  END;



(* The zombie inteligence... or someting. *)
  PROCEDURE TZombie.Update;

  (* Helper function to know if it is near the player. *)
    FUNCTION IsNearPlayer: BOOLEAN; INLINE;
    BEGIN
      IsNearPlayer := Distance2 (
	SpriteList[ThePlayer.SpriteIndex].X, SpriteList[ThePlayer.SpriteIndex].Y,
	SpriteList[fSprIndex].X, SpriteList[fSprIndex].Y) <= MAX_CAN_SEE_DISTANCE2;
    END;

  VAR
    Dir: INTEGER;
  BEGIN
  { Each status has different process. }
	TODO
    CASE fStatus OF
      aaStop, aaScentry:
      BEGIN
	Dir := 0;
	IF IsNearPlayer THEN
	BEGIN
	  fStatus := aaWaiting;
	  Dir := LookAt (
	    SpriteList[fSprIndex].X, SpriteList[fSprIndex].Y,
	    SpriteList[ThePlayer.SpriteIndex].X, SpriteList[ThePlayer.SpriteIndex].Y
	    );
	END
	ELSE BEGIN
	{ Random walking. }
	  IF Random (20000) < 1 THEN
	    Dir := Random (8) + 1;
	  IF Random (2000) < 1 THEN
	    fStatus := aaWalking;
	END;
	IF (Dir <> 0) AND (fDirection <> Dir) THEN
	BEGIN
	  fDirection := Dir;
	  Counter := 0;
	  SpriteList[fSprIndex].Picture := fPictures.Picture[Dir, fFrame];
	END;
      END;
      aaWaiting:
      BEGIN
	INC (Counter);
	IF Counter >= 40 THEN
	BEGIN
	  Counter := 0;
	{ Check if it's near the player. }
	  IF IsNearPlayer THEN
	    fDirection := LookAt (
	      SpriteList[fSprIndex].X, SpriteList[fSprIndex].Y,
	      SpriteList[ThePlayer.SpriteIndex].X, SpriteList[ThePlayer.SpriteIndex].Y
	    );
	  IF IsNearPlayer OR (Random (3) = 1) THEN
	    fStatus := aaWalking
	  ELSE
	    fStatus := aaStop;
	END;
      END;
      aaWalking:
      BEGIN
	IF Counter MOD 2 = 1 THEN
	BEGIN
	{ Checks walking direction. }
	  IF (fDirection IN RS_G_W)
	  AND NOT CheckLeftCollision (fSprIndex) THEN
	    DEC (SpriteList[fSprIndex].x);
	  IF (fDirection IN RS_G_E)
	  AND NOT CheckRightCollision (fSprIndex) THEN
	    INC (SpriteList[fSprIndex].x);
	  IF (fDirection IN RS_G_N)
	  AND NOT CheckCeilCollision (fSprIndex) THEN
	    DEC (SpriteList[fSprIndex].y);
	  IF (fDirection IN RS_G_S)
	  AND NOT CheckGroundCollision (fSprIndex) THEN
	    INC (SpriteList[fSprIndex].y);
	END;
      { Checks if the sprite should be changed. }
	INC (Counter);
	IF Counter >= 40 THEN
	BEGIN
	  fStatus := aaWaiting;
	  Counter := 0;
	END;
	SpriteList[fSprIndex].Picture := fPictures.Picture[fDirection, (Counter DIV 10) + 1];
      END;
    END;
  END;

END.
